Imports System.ComponentModel

''' <summary>
''' Eine eigene MessageBox, die auch detaillierte Hinweise anzeigen kann.
''' </summary>
''' <remarks>
''' <example>
''' Dim frm As CNDialoge.frmMessageBox = New CNDialoge.frmMessageBox()
''' 
''' frm.Fenstertitel = "Frage"
''' frm.Text = "Mchten Sie das wirklich tun?"
''' frm.OKButtonText = "Gaanz langer Text"
''' frm.AbbrechenButtonText = "Ist nicht sichtbar"
''' frm.ShowOKButton = True
''' frm.ShowAbbrechenButton = False
''' frm.DefaultButton = CNDialoge.eMessageBoxStandardButton.Abbrechen
''' frm.MessageIcon = Drawing.SystemIcons.Question
''' frm.Icon = Drawing.SystemIcons.Application
''' frm.DetailText = "Hallo"
''' 
''' frm.ShowDialog()
''' </example>
''' </remarks>
Public Class frmMessageBox

#Region "Variablendefinitionen"

    ''' <summary>
    ''' Kennzeichen, ob aktuell die Details expandiert (sichtbar) sind.
    ''' </summary>
    Private m_DetailsExpandiert As Boolean = False

    ''' <summary>
    ''' Speichert die initiale Hhe des Splitpanels, das ein- bzw ausgeklappt
    ''' werden knnen soll.
    ''' </summary>
    Private m_SplitPanel2Height As Integer = 0

    ''' <summary>
    ''' Icon, das neben der Nachricht angezeigt wird.
    ''' </summary>
    Private m_MessageIcon As System.Drawing.Icon = Nothing

    ''' <summary>
    ''' Fenstertitel fr das Nachrichtenfenster.
    ''' </summary>
    Private m_Fenstertitel As String = String.Empty

    ''' <summary>
    ''' Der aktuell hinterlegte Standardbutton.
    ''' </summary>
    Private m_StandardButton As eMessageBoxStandardButton = 0

#End Region

#Region "Eigenschaften"

    ''' <summary>
    ''' Eigenschaft, die den Detailtext zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktuell hinterlegter Detailtext.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den Detailtext zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property DetailText() As String
        Get
            GetThreadSafeObject()
            Return m_txtDetails.Text
        End Get
        Set(ByVal value As String)
            GetThreadSafeObject()
            m_txtDetails.Text = value
            OnDetailTextChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die das anzuzeigende Icon zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktuell hinterlegtes Icon.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die das anzuzeigende Icon zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property MessageIcon() As System.Drawing.Icon
        Get
            GetThreadSafeObject()
            Return m_MessageIcon
        End Get
        Set(ByVal value As System.Drawing.Icon)
            m_MessageIcon = value
            OnMessageIconChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den anzuzeigenden Text zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktuell hinterlegter Anzeigetext.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den anzuzeigenden Text zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Overrides Property Text() As String
        Get
            If (m_lblTextHeader Is Nothing) Then
                Return String.Empty
            End If
            GetThreadSafeObject()
            Return m_lblTextHeader.Text
        End Get
        Set(ByVal value As String)
            GetThreadSafeObject()
            m_lblTextHeader.Text = value
            OnTextChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den anzuzeigenden Fenstertiteltext zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktueller Fenstertitel.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den anzuzeigenden Fenstertiteltext zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property Fenstertitel() As String
        Get
            GetThreadSafeObject()
            Return m_Fenstertitel
        End Get
        Set(ByVal value As String)
            m_Fenstertitel = value
            OnFenstertitelChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den anzuzeigenden Text des OK Buttons zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktueller OK Button Text.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den anzuzeigenden Text des OK Buttons zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property OKButtonText() As String
        Get
            GetThreadSafeObject()
            Return m_btnOK.Text
        End Get
        Set(ByVal value As String)
            GetThreadSafeObject()
            m_btnOK.Text = value
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den anzuzeigenden Text des Abbrechen Buttons zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktueller Abbrechen Button Text.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den anzuzeigenden Text des Abbrechen Buttons zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property AbbrechenButtonText() As String
        Get
            GetThreadSafeObject()
            Return m_btnAbbrechen.Text
        End Get
        Set(ByVal value As String)
            GetThreadSafeObject()
            m_btnAbbrechen.Text = value
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den Anzeigezustand des OK Buttons zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>True, wenn der OK Button gezeigt wird, andernfalls False.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den Anzeigezustand des OK Buttons zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property ShowOKButton() As Boolean
        Get
            GetThreadSafeObject()
            Return m_btnOK.Visible
        End Get
        Set(ByVal value As Boolean)
            GetThreadSafeObject()
            m_btnOK.Visible = value
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den Anzeigezustand des Abbrechen Buttons zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>True, wenn der Abbrechen Button gezeigt wird, andernfalls False.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den Anzeigezustand des Abbrechen Buttons zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property ShowAbbrechenButton() As Boolean
        Get
            GetThreadSafeObject()
            Return m_btnAbbrechen.Visible
        End Get
        Set(ByVal value As Boolean)
            GetThreadSafeObject()
            m_btnAbbrechen.Visible = value
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den Index des Standardbuttons zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Index des aktuellen Standardbutton.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den Index des Standardbuttons zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Property DefaultButton() As eMessageBoxStandardButton
        Get
            Return m_StandardButton
        End Get
        Set(ByVal value As eMessageBoxStandardButton)
            m_StandardButton = value
            SetzeDefaultButton(value)
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die den Anzeigestatus des Fenstericons zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>True, wenn aktuell das Fenstericon angezeigt wird, andernfalls False.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die den Anzeigestatus des Fenstericons zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Shadows Property ShowIcon() As Boolean
        Get
            GetThreadSafeObject()
            Return MyBase.ShowIcon
        End Get
        Set(ByVal value As Boolean)
            If MyBase.ShowIcon <> value Then
                MyBase.ShowIcon = value
                OnShowIconChanged(EventArgs.Empty)
            End If
        End Set
    End Property

    ''' <summary>
    ''' Eigenschaft, die das Fenstericon zurckgibt oder setzt.
    ''' </summary>
    ''' <returns>Aktuell hinterlegtes Fenstericon.</returns>
    <DefaultValue("")> _
    <Category("CidaNova")> _
    <Description("Eigenschaft, die das Fenstericon zurckgibt oder setzt.")> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <Browsable(True)> _
    Public Shadows Property Icon() As Drawing.Icon
        Get
            GetThreadSafeObject()
            Return MyBase.Icon
        End Get
        Set(ByVal value As Drawing.Icon)
            MyBase.Icon = value
            ShowIcon = Not value Is Nothing
        End Set
    End Property

#End Region

#Region "Konstruktor"

    ''' <summary>
    ''' Erstellt eine neue Instanz des Hinweisfensters.
    ''' </summary>
    Public Sub New()
        InitializeComponent()
        Initialisierung()
    End Sub

    ''' <summary>
    ''' Initialisierungen fr die Objekterstellung.
    ''' </summary>
    Private Sub Initialisierung()

        ' Die initiale Hhe des Splitpanels speichern
        m_SplitPanel2Height = m_spltDetails.Height - m_spltDetails.SplitterDistance

    End Sub

#End Region

#Region "Events"

    ''' <summary>
    ''' Ereignis, das bei einer Vernderung des Detailtextes ausgelst wird.
    ''' </summary>
    Public Event DetailTextChanged As EventHandler

    ''' <summary>
    ''' Ereignis, das bei einer Vernderung des Nachrichtenicons ausgelst wird.
    ''' </summary>
    Public Event MessageIconChanged As EventHandler

    ''' <summary>
    ''' Ereignis, das bei einer Vernderung des Anzeigetextes ausgelst wird.
    ''' </summary>
    Public Shadows Event TextChanged As EventHandler

    ''' <summary>
    ''' Ereignis, das bei einer Vernderung des Fenstertitels ausgelst wird.
    ''' </summary>
    Public Event FenstertitelChanged As EventHandler

    ''' <summary>
    ''' Ereignis, das bei einer Vernderung des Anzeigestatus vom Fenstericon ausgelst wird.
    ''' </summary>
    Public Event ShowIconChanged As EventHandler

    ''' <summary>
    ''' Behandelt das Load Ereignis des <see cref="frmMessageBox"></see> Fensters.
    ''' </summary>
    ''' <param name="sender">Die Quelle des Ereignisauslsers.</param>
    ''' <param name="e">Die bergebenen Ereignisparameter.</param>
    Private Sub frmMessageBox_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Einklappen der Detailinformationen
        DetailsEinklappen()

    End Sub

    ''' <summary>
    ''' Behandelt das Shown Ereignis des <see cref="frmMessageBox"></see> Fensters.
    ''' </summary>
    ''' <param name="sender">Die Quelle des Ereignisauslsers.</param>
    ''' <param name="e">Die bergebenen Ereignisparameter.</param>
    Private Sub frmMessageBox_Shown(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Shown

        ' Focus des aktuell hinterlegten Standardbuttons forcieren.
        ' Vor dem anzeigen des Fensters kann die Fokusierung nicht klappen.
        SetzeDefaultButton(m_StandardButton)

    End Sub

    ''' <summary>
    ''' Behandelt das Klick-Ereignis des <see cref="m_btnOK"></see> Buttons.
    ''' </summary>
    ''' <param name="sender">Die Quelle des Ereignisauslsers.</param>
    ''' <param name="e">Die bergebenen Ereignisparameter.</param>
    Private Sub m_btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_btnOK.Click

        ' Dialogrckgabewert festlegen
        Me.DialogResult = Windows.Forms.DialogResult.OK

        ' Dialog schlieen
        Me.Close()

    End Sub

    ''' <summary>
    ''' Behandelt das Klick-Ereignis des <see cref="m_btnAbbrechen"></see> Buttons.
    ''' </summary>
    ''' <param name="sender">Die Quelle des Ereignisauslsers.</param>
    ''' <param name="e">Die bergebenen Ereignisparameter.</param>
    Private Sub m_btnAbbrechen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_btnAbbrechen.Click

        ' Dialogrckgabewert festlegen
        Me.DialogResult = Windows.Forms.DialogResult.Cancel

        ' Dialog schlieen
        Me.Close()

    End Sub

    ''' <summary>
    ''' Behandelt das Klick-Ereignis des <see cref="m_btnDetails"></see> Buttons.
    ''' </summary>
    ''' <param name="sender">Die Quelle des Ereignisauslsers.</param>
    ''' <param name="e">Die bergebenen Ereignisparameter.</param>
    Private Sub m_btnDetails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_btnDetails.Click

        ' Details Ein- / Ausklappen
        If m_DetailsExpandiert Then
            DetailsEinklappen()
        Else
            DetailsAusklappen()
        End If

    End Sub

#End Region

#Region "Private Methoden"

    ''' <summary>
    ''' Methode, die die Detailinformationen einklappt und entsprechende Controls
    ''' de- / aktiviert.
    ''' </summary>
    Private Sub DetailsEinklappen()

        ' Fensterhhe verkleinern und Textspringen verhindern
        Me.Height -= m_SplitPanel2Height
        m_spltDetails.Panel2Collapsed = True

        ' Button anpassen
        m_btnDetails.Text = "&Details >>"

        ' Zustandsvariable anpassen
        m_DetailsExpandiert = False

    End Sub

    ''' <summary>
    ''' Methode, die die Detailinformationen ausklappt und entsprechende Controls
    ''' de- / aktiviert.
    ''' </summary>
    Private Sub DetailsAusklappen()

        ' Fensterhhe vergrern und Textspringen verhindern
        Me.Height += m_SplitPanel2Height
        m_spltDetails.Panel2Collapsed = False

        ' Button anpassen
        m_btnDetails.Text = "&Details <<"

        ' Zustandsvariable anpassen
        m_DetailsExpandiert = True

    End Sub

    ''' <summary>
    ''' Methode, die den Standardbutton setzt.
    ''' </summary>
    ''' <param name="p_Val">Angabe, welcher Standardbutton gesetzt werden soll.</param>
    Private Sub SetzeDefaultButton(ByVal p_Val As eMessageBoxStandardButton)

        ' Je nach dem Wert entsprechenden Button den Fokus erteilen
        Select Case p_Val
            Case eMessageBoxStandardButton.OK
                m_btnOK.Focus()
            Case eMessageBoxStandardButton.Abbrechen
                m_btnAbbrechen.Focus()
            Case eMessageBoxStandardButton.Kein
                Me.Focus()
        End Select

    End Sub

    ''' <summary>
    ''' Methode, die den Fenstertitel setzt.
    ''' </summary>
    ''' <param name="p_Titel">Titel, der gesetzt werden soll.</param>
    ''' <remarks>Diese Methode ist threadsicher.</remarks>
    Private Sub SetzeFenstertitel(ByVal p_Titel As String)

        ' lokale Variablen
        Dim dele As EinfacherParameterDelegat(Of String) = New EinfacherParameterDelegat(Of String)(AddressOf SetzeFenstertitel)

        ' Threadsicherheit berprfen
        If (MyBase.InvokeRequired) Then
            MyBase.Invoke(dele, p_Titel)
        Else
            MyBase.Text = p_Titel
        End If

    End Sub

    ''' <summary>
    ''' Methode, die ein threadsicheres Objekt zurckliefert.
    ''' </summary>
    Private Sub GetThreadSafeObject()

        ' lokale Variablen
        Dim dele As VoidDelegate = New VoidDelegate(AddressOf GetThreadSafeObject)

        ' Threadsicherheit berprfen und ggf. ber einen Delegaten aufrufen
        If (Me.InvokeRequired) Then
            Me.Invoke(dele)
        End If

    End Sub

#End Region

#Region "Geschtzte Methoden"

    ''' <summary>
    ''' Methode, die das Ereignis <see cref="E:DetailTextChanged"></see> auslst.
    ''' </summary>
    ''' <param name="p_EventArgs">Ereignisparameter, die bergeben werden.</param>
    Protected Overridable Sub OnDetailTextChanged(ByVal p_EventArgs As EventArgs)

        ' De- / Aktiviert Detail Button
        GetThreadSafeObject()
        m_btnDetails.Enabled = Not m_txtDetails.Text.Equals(String.Empty)

        ' Ereignis auslsen
        RaiseEvent DetailTextChanged(Me, p_EventArgs)

    End Sub

    ''' <summary>
    ''' Methode, die das Ereignis <see cref="E:MessageIconChanged"></see> auslst.
    ''' </summary>
    ''' <param name="p_EventArgs">Ereignisparameter, die bergeben werden.</param>
    Protected Overridable Sub OnMessageIconChanged(ByVal p_EventArgs As EventArgs)

        ' lokale Variablen
        GetThreadSafeObject()
        Dim picBox As Windows.Forms.PictureBox = m_picIcon

        ' Icon der PictureBox zuweisen
        If m_MessageIcon Is Nothing Then
            picBox.Image = Nothing
            picBox.Visible = False
        Else
            picBox.Image = m_MessageIcon.ToBitmap()
            picBox.Visible = True
        End If

        ' Fenster aktualisieren lassen
        Me.Invalidate()

        ' Ereignis auslsen
        RaiseEvent MessageIconChanged(Me, p_EventArgs)

    End Sub

    ''' <summary>
    ''' Methode, die das Ereignis <see cref="E:FenstertitelChanged" /> auslst.
    ''' </summary>
    ''' <param name="p_EventArgs">Ereignisparameter, die bergeben werden.</param>
    Protected Overridable Sub OnFenstertitelChanged(ByVal p_EventArgs As System.EventArgs)

        ' Fenstertitel ber die Basisklasse festlegen
        SetzeFenstertitel(m_Fenstertitel)

        ' Ereignis auslsen
        RaiseEvent FenstertitelChanged(Me, p_EventArgs)

    End Sub

    ''' <summary>
    ''' Methode, die das Ereignis <see cref="E:TextChanged" /> auslst.
    ''' </summary>
    ''' <param name="p_EventArgs">Ereignisparameter, die bergeben werden.</param>
    Protected Overrides Sub OnTextChanged(ByVal p_EventArgs As System.EventArgs)

        ' Ereignis auslsen
        RaiseEvent TextChanged(Me, p_EventArgs)

    End Sub

    ''' <summary>
    ''' Methode, die das Ereignis <see cref="E:ShowIconChanged"></see> auslst.
    ''' </summary>
    ''' <param name="p_EventArgs">Ereignisparameter, die bergeben werden.</param>
    Protected Overridable Sub OnShowIconChanged(ByVal p_EventArgs As System.EventArgs)

        ' Ereignis auslsen
        RaiseEvent ShowIconChanged(Me, p_EventArgs)

    End Sub


#End Region

End Class